package de.hottis.MeasurementCollector;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentLinkedQueue;

import org.openmuc.jmbus.DataRecord;
import org.openmuc.jmbus.MBusMessage;
import org.openmuc.jmbus.VariableDataStructure;

import com.json.parsers.JSONParser;
import com.json.parsers.JsonParserFactory;

public class MBusParser extends AMessageParser {

	static final String TOPIC = "IoT/Measurement/MeterbusHub";

	private final JSONParser jsonParser;
	private final HottisFourChannelThermometer hottisFourChannelThermometer;
	private final FinderOnePhasePowerMeter finderOnePhasePowerMeter;
	private final FinderThreePhasePowerMeter finderThreePhasePowerMeter;
	
	public MBusParser(ConcurrentLinkedQueue<List<ADataObject>> queue) {
		super(TOPIC, queue);
		JsonParserFactory jsonParserFactory = JsonParserFactory.getInstance();
		jsonParser = jsonParserFactory.newJsonParser();
		hottisFourChannelThermometer = new HottisFourChannelThermometer();
		finderOnePhasePowerMeter = new FinderOnePhasePowerMeter();
		finderThreePhasePowerMeter = new FinderThreePhasePowerMeter();
	}

	@Override
	public void execute(LocalDateTime timestamp, String msgPayload) {
		try {
			Map payloadMap = jsonParser.parseJson(msgPayload); 
			String name = (String)(((Map)(payloadMap.get("metadata"))).get("name"));
			String mbusMsgTxt = (String)(((Map)(payloadMap.get("data"))).get("telegram"));
			String [] octetsTxt = mbusMsgTxt.split(" ");
			byte [] octets = new byte[octetsTxt.length];
			for (int i = 0; i < octetsTxt.length; i++) {
				octets[i] = (byte)(Integer.parseInt(octetsTxt[i], 16) & 0xff);
			}
			
			MBusMessage mbusMsg = MBusMessage.decode(octets, octets.length);
			VariableDataStructure variableDataStructure = mbusMsg.getVariableDataResponse();
			variableDataStructure.decode();
			List<DataRecord> dataRecords = variableDataStructure.getDataRecords();

			//for (DataRecord dataRecord : dataRecords) {
			//	System.out.println(dataRecord.getScaledDataValue() + " " + dataRecord.getUnit().getUnit());
			//}
			
			List<ADataObject> measurementItems;
			switch (name) {
			case "thermom.":
				measurementItems = hottisFourChannelThermometer.parse(timestamp, name, dataRecords);
				break;
			case "light":
			case "computer":
			case "freezer":
			case "dryer":
			case "laundry":
			case "dishwasher":
				measurementItems = finderOnePhasePowerMeter.parse(timestamp, name.substring(0,1).toUpperCase() + name.substring(1).toLowerCase(), dataRecords);
				break;
			case "electricity":
				measurementItems = finderThreePhasePowerMeter.parse(timestamp, "Total", dataRecords);
				break;
			default:
				System.out.println("unknown name");
				measurementItems = null;
			}
			
			for (ADataObject ado : measurementItems) {
				System.out.println(ado);
			}
		} catch (Exception e) {
			System.out.println("Exception when handling mbus message: " + e);
		}

	}

}
